//Example of using non-blocking, listening, multithreading, graceful shutdown
//Full debug information provided in binary to test for memory leaks using e.g. MemProof
//Connect to 127.0.0.1 port 2876 using telnet.exe to test
//
//I strongly recommand against using non-blocking sockets for normal server applications,
//see 'Using threads' in Delphi help.
//
// http://www.ward.nu/computer/tsock

unit unonblo1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Sock, StdCtrls;

type
  TMainForm = class(TForm)
    MasterSock: TSock;
    LogBox: TListBox;
    procedure MasterSockAutoAccept(Sender: TObject; NewSock: TSock);
    procedure MasterSockInfo(Sender: TObject; SocketInfo: TSocketInfo;
      Msg: String);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure MasterSockRead(Sender: TObject; Count: Integer);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

procedure TMainForm.MasterSockAutoAccept(Sender: TObject; NewSock: TSock);
begin
  LogBox.Items.Insert(0, 'Misc: Remote host is '+NewSock.RemoteHost);
  try
    NewSock.SendLine('Welcome to '+GetLocalHostname+'!');
  except
    on E:ESockException do LogBox.Items.Insert(0, 'Exception: '+E.Message);
  end;
end;

procedure TMainForm.MasterSockInfo(Sender: TObject; SocketInfo: TSocketInfo;
  Msg: String);
begin
  LogBox.Items.Insert(0, 'OnInfo: '+Msg);
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
var
  I: Integer;
  EnumComp: TComponent;
begin
  MasterSock.StopListen;
  LogBox.Items.Insert(0, 'Misc: Terminating open connections... Please wait...');
  for I:=MasterSock.ComponentCount-1 downto 0 do
  begin
    EnumComp:=MasterSock.Components[I];
    if (EnumComp is TSock) then
    begin
      try
        (EnumComp as TSock).SendLine('Server shutting down... Bye!');
      except
        on E:ESockException do LogBox.Items.Insert(0, 'Exception: '+E.Message);
      end;
      (EnumComp as TSock).Close;
    end;
  end;
  for I:=0 to 100 do
  begin
    Application.ProcessMessages;
    Sleep(10);
  end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  LogBox.Items.Insert(0, 'Misc: '+WSDescription+' ('+WSSystemStatus+')');
end;

procedure TMainForm.MasterSockRead(Sender: TObject; Count: Integer);
var
  Command: String;
  NewSock: TSock;
begin
  NewSock:=(Sender as TSock);
  try
    Command:=NewSock.ReceiveLine;
    if (Length(Command)>0) then
    begin
      LogBox.Items.Insert(0, 'Receive: '+Command);
      if (UpperCase(Command)='DATE') then
        NewSock.SendLine(DateTimeToStr(Now))
      else if (UpperCase(Command)='QUIT') then
        NewSock.Close
      else
        NewSock.SendLine('Unknown command. Valid commands are: DATE , QUIT');
    end;
  except
    on E:ESockException do LogBox.Items.Insert(0, 'Exception: '+E.Message);
  end;
end;

end.
